public class CelebrityProblem {
    private int[][] matrix;

    // Constructor
    public CelebrityProblem(int[][] matrix) {
        this.matrix = matrix;
    }

    // Returns true if person i knows person j, false otherwise
    private boolean knows(int i, int j) {
        return matrix[i][j] == 1;
    }

    /*
     * Find the celebrity in the group
     * Celebrity is a person, which is known by everyone, but doesn't know anyone themselves
     */
    public int findCelebrity(int n) {
        int celebrityCandidate = 0;

        // Find the candidate for celebrity
        for (int i = 1; i < n; i++) {
            if (knows(celebrityCandidate, i)) {
                celebrityCandidate = i;
            }
        }

        // Check if the candidate is a celebrity
        for (int i = 0; i < n; i++) {
            if (i != celebrityCandidate && (knows(celebrityCandidate, i) || !knows(i, celebrityCandidate))) {
                return -1; // No celebrity found
            }
        }

        return celebrityCandidate;
    }

    public static void main(String[] args) {
        int[][] matrix = {
                {0, 1, 1, 0},
                {0, 0, 1, 0},
                {0, 0, 0, 0},
                {0, 1, 1, 0}
        };

        CelebrityProblem cp = new CelebrityProblem(matrix);
        int celebrity = cp.findCelebrity(4);

        if (celebrity == -1) {
            System.out.println("No celebrity found.");
        } else {
            System.out.println("Celebrity found: Person " + celebrity);
        }
    }
}
