import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class CelebrityProblemTest {
    private CelebrityProblem cp;

    @BeforeEach
    public void setup() {
        int[][] matrix = {
                {0, 1, 1, 0},
                {0, 0, 1, 0},
                {0, 0, 0, 0},
                {0, 1, 1, 0}
        };
        cp = new CelebrityProblem(matrix);
    }

    // Generate a matrix with the given number of people (n) and the specified celebrity
    private int[][] generateMatrixWithCelebrity(int n, int celebrity) {
        int[][] matrix = new int[n][n];

        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                if (i != celebrity && j == celebrity) {
                    matrix[i][j] = 1;
                } else {
                    matrix[i][j] = 0;
                }
            }
        }

        return matrix;
    }

    @Test
    public void testFindCelebrity() {
        int celebrity = cp.findCelebrity(4);
        assertEquals(2, celebrity);
    }

    @Test
    public void testNoCelebrity() {
        int[][] matrix = {
                {0, 1, 0, 0},
                {1, 0, 1, 0},
                {0, 0, 0, 0},
                {1, 1, 1, 0}
        };
        cp = new CelebrityProblem(matrix);
        int celebrity = cp.findCelebrity(4);
        assertEquals(-1, celebrity);
    }

    @Test
    public void testAllCelebrities() {
        int[][] matrix = {
                {0, 0, 0, 0},
                {1, 0, 0, 0},
                {1, 1, 0, 0},
                {1, 1, 1, 0}
        };
        cp = new CelebrityProblem(matrix);
        int celebrity = cp.findCelebrity(4);
        assertEquals(0, celebrity);
    }

    @Test
    public void testNoConnections() {
        int[][] matrix = {
                {0, 0, 0, 0},
                {0, 0, 0, 0},
                {0, 0, 0, 0},
                {0, 0, 0, 0}
        };
        cp = new CelebrityProblem(matrix);
        int celebrity = cp.findCelebrity(4);
        assertEquals(-1, celebrity);
    }

    @Test
    public void testCelebrity55() {
        int[][] matrix = generateMatrixWithCelebrity(100, 55);
        cp = new CelebrityProblem(matrix);
        int celebrity = cp.findCelebrity(100);
        assertEquals(55, celebrity);
    }
}
