import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;

public class PolynomialTest {

    @Test
    public void testAddition() {
        Polynomial p1 = new Polynomial(new double[]{1, 2, 3});
        Polynomial p2 = new Polynomial(new double[]{2, -1, 1});
        Polynomial sum = p1.add(p2);

        assertArrayEquals(new double[]{3, 1, 4}, sum.coefficients);
    }

    @Test
    public void testSubtraction() {
        Polynomial p1 = new Polynomial(new double[]{1, 2, 3});
        Polynomial p2 = new Polynomial(new double[]{2, -1, 1});
        Polynomial difference = p1.subtract(p2);

        assertArrayEquals(new double[]{-1, 3, 2}, difference.coefficients);
    }

    @Test
    public void testMultiplication() {
        Polynomial p1 = new Polynomial(new double[]{1, 2, 3});
        Polynomial p2 = new Polynomial(new double[]{2, -1, 1});
        Polynomial product = p1.multiply(p2);

        assertArrayEquals(new double[]{2, 3, 5, -1, 3}, product.coefficients);
    }

    @Test
    public void testEvaluation() {
        Polynomial p = new Polynomial(new double[]{1, 2, 3}); // 3x^2 + 2x + 1
        double value = p.evaluate(2); // 3(2)^2 + 2(2) + 1 = 12 + 4 + 1 = 17

        assertEquals(17, value);
    }

    @Test
    public void testDerivative() {
        Polynomial p = new Polynomial(new double[]{1, 2, 3}); // 3x^2 + 2x + 1
        Polynomial derivative = p.derivative(); // 6x + 2

        assertArrayEquals(new double[]{2, 6}, derivative.coefficients);
    }

    @Test
    public void testIntegral() {
        Polynomial p = new Polynomial(new double[]{1, 2, 3}); // 3x^2 + 2x + 1
        Polynomial integral = p.integral(); // x^3 + x^2 + x + C

        assertArrayEquals(new double[]{0, 1, 1, 1}, integral.coefficients);
    }
}
